import 'package:solana_rpc_sdk_flutter/model/rpc_response.dart';

class GetTransaction extends BaseModel {
  int? blockTime;
  Meta? meta;
  int? slot;
  Transaction? transaction;

  GetTransaction({this.blockTime, this.meta, this.slot, this.transaction});

  GetTransaction.fromJson(Map<String, dynamic> json) {
    blockTime = json['blockTime'];
    meta = json['meta'] != null ? Meta.fromJson(json['meta']) : null;
    slot = json['slot'];
    transaction = json['transaction'] != null
        ? Transaction.fromJson(json['transaction'])
        : null;
  }

  @override
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['blockTime'] = blockTime;
    if (meta != null) {
      data['meta'] = meta!.toJson();
    }
    data['slot'] = slot;
    if (transaction != null) {
      data['transaction'] = transaction!.toJson();
    }
    return data;
  }
}

class Meta {
  Map<String, dynamic>? err;
  int? fee;
  List<dynamic>? innerInstructions;
  List<String>? logMessages;
  List<int>? postBalances;
  List<int>? postTokenBalances;
  List<int>? preBalances;
  List<int>? preTokenBalances;
  List<dynamic>? rewards;
  Status? status;

  Meta(
      {this.err,
      this.fee,
      this.innerInstructions,
      this.logMessages,
      this.postBalances,
      this.postTokenBalances,
      this.preBalances,
      this.preTokenBalances,
      this.rewards,
      this.status});

  Meta.fromJson(Map<String, dynamic> json) {
    err = json['err'];
    fee = json['fee'];
    if (json['innerInstructions'] != null) {
      innerInstructions = <dynamic>[];
      json['innerInstructions'].forEach((v) {
        innerInstructions!.add(v);
      });
    }
    logMessages = json['logMessages']?.cast<String>();
    postBalances = json['postBalances']?.cast<int>();
    if (json['postTokenBalances'] != null) {
      postTokenBalances = <int>[];
      json['postTokenBalances'].forEach((v) {
        postTokenBalances!.add(v);
      });
    }
    preBalances = json['preBalances'].cast<int>();
    if (json['preTokenBalances'] != null) {
      preTokenBalances = <int>[];
      json['preTokenBalances'].forEach((v) {
        preTokenBalances!.add(v);
      });
    }
    if (json['rewards'] != null) {
      rewards = <dynamic>[];
      json['rewards'].forEach((v) {
        rewards!.add(v);
      });
    }
    status = json['status'] != null ? Status.fromJson(json['status']) : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['err'] = err;
    data['fee'] = fee;
    if (innerInstructions != null) {
      data['innerInstructions'] =
          innerInstructions!.map((v) => v.toJson()).toList();
    }
    data['logMessages'] = logMessages;
    data['postBalances'] = postBalances;
    if (postTokenBalances != null) {
      data['postTokenBalances'] = postTokenBalances!.map((v) => v).toList();
    }
    data['preBalances'] = preBalances;
    if (preTokenBalances != null) {
      data['preTokenBalances'] = preTokenBalances!.map((v) => v).toList();
    }
    if (rewards != null) {
      data['rewards'] = rewards!.map((v) => v.toJson()).toList();
    }
    if (status != null) {
      data['status'] = status!.toJson();
    }
    return data;
  }
}

class Status {
  Null? ok;

  Status({this.ok});

  Status.fromJson(Map<String, dynamic> json) {
    ok = json['Ok'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Ok'] = ok;
    return data;
  }
}

class Transaction {
  Message? message;
  List<String>? signatures;

  Transaction({this.message, this.signatures});

  Transaction.fromJson(Map<String, dynamic> json) {
    message =
        json['message'] != null ? Message.fromJson(json['message']) : null;
    signatures = json['signatures'].cast<String>();
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    if (message != null) {
      data['message'] = message!.toJson();
    }
    data['signatures'] = signatures;
    return data;
  }
}

class Message {
  List<AccountKeys>? accountKeys;
  List<Instructions>? instructions;
  String? recentBlockhash;

  Message({this.accountKeys, this.instructions, this.recentBlockhash});

  Message.fromJson(Map<String, dynamic> json) {
    if (json['accountKeys'] != null) {
      accountKeys = <AccountKeys>[];
      json['accountKeys'].forEach((v) {
        accountKeys!.add(AccountKeys.fromJson(v));
      });
    }
    if (json['instructions'] != null) {
      instructions = <Instructions>[];
      json['instructions'].forEach((v) {
        instructions!.add(Instructions.fromJson(v));
      });
    }
    recentBlockhash = json['recentBlockhash'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    if (accountKeys != null) {
      data['accountKeys'] = accountKeys!.map((v) => v.toJson()).toList();
    }
    if (instructions != null) {
      data['instructions'] = instructions!.map((v) => v.toJson()).toList();
    }
    data['recentBlockhash'] = recentBlockhash;
    return data;
  }
}

class AccountKeys {
  String? pubkey;
  bool? signer;
  bool? writable;

  AccountKeys({this.pubkey, this.signer, this.writable});

  AccountKeys.fromJson(Map<String, dynamic> json) {
    pubkey = json['pubkey'];
    signer = json['signer'];
    writable = json['writable'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['pubkey'] = pubkey;
    data['signer'] = signer;
    data['writable'] = writable;
    return data;
  }
}

class Instructions {
  List<String>? accounts;
  String? data;
  String? programId;

  Instructions({this.accounts, this.data, this.programId});

  Instructions.fromJson(Map<String, dynamic> json) {
    accounts = json['accounts'].cast<String>();
    data = json['data'];
    programId = json['programId'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['accounts'] = accounts;
    data['data'] = this.data;
    data['programId'] = programId;
    return data;
  }
}
