import 'dart:ffi';

typedef CallbackToObject<T> = T Function(Map<String, dynamic> value);
typedef CallbackToJson<T> = dynamic Function(T value);

class RPCResponse<T> extends BaseModel {
  String? jsonrpc;
  T? result;
  int? id;

  RPCResponse({this.jsonrpc, this.result, this.id});

  @override
  RPCResponse.fromJson(Map<String, dynamic> json,
      {CallbackToObject<T>? onData}) {
    jsonrpc = json['jsonrpc'];
    id = json['id'];

    if (json['result'] is int ||
        json['result'] is double ||
        json['result'] is bool ||
        json['result'] is String) {
      result = json['result'];
    } else {
      result = onData!(json['result']);
    }
  }

  @override
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['jsonrpc'] = jsonrpc;
    if (result != null) {
      if (result is BaseModel) {
        data['result'] = (result as BaseModel).toJson();
      } else {
        data['result'] = result;
      }
    }
    data['id'] = id;
    return data;
  }
}

abstract class BaseModel {
  BaseModel();

  BaseModel.fromJson(Map<String, dynamic> json) {}

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    return data;
  }
}
