import 'package:solana_rpc_sdk_flutter/model/rpc_response.dart';

class TokenAccountsByOwner extends BaseModel {
  Context? context;
  List<Value>? value;

  TokenAccountsByOwner({this.context, this.value});

  @override
  TokenAccountsByOwner.fromJson(Map<String, dynamic> json) {
    context =
        json['context'] != null ? Context.fromJson(json['context']) : null;
    if (json['value'] != null) {
      value = <Value>[];
      json['value'].forEach((v) {
        value!.add(Value.fromJson(v));
      });
    }
  }

  @override
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    if (context != null) {
      data['context'] = context!.toJson();
    }
    if (value != null) {
      data['value'] = value!.map((v) => v.toJson()).toList();
    }
    return data;
  }
}

class Context {
  int? slot;

  Context({this.slot});

  Context.fromJson(Map<String, dynamic> json) {
    slot = json['slot'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['slot'] = slot;
    return data;
  }
}

class Value {
  Account? account;
  String? pubkey;

  Value({this.account, this.pubkey});

  Value.fromJson(Map<String, dynamic> json) {
    account =
        json['account'] != null ? Account.fromJson(json['account']) : null;
    pubkey = json['pubkey'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    if (account != null) {
      data['account'] = account!.toJson();
    }
    data['pubkey'] = pubkey;
    return data;
  }
}

class Account {
  Data? data;
  bool? executable;
  int? lamports;
  String? owner;
  int? rentEpoch;

  Account(
      {this.data, this.executable, this.lamports, this.owner, this.rentEpoch});

  Account.fromJson(Map<String, dynamic> json) {
    data = json['data'] != null ? Data.fromJson(json['data']) : null;
    executable = json['executable'];
    lamports = json['lamports'];
    owner = json['owner'];
    rentEpoch = json['rentEpoch'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    if (this.data != null) {
      data['data'] = this.data!.toJson();
    }
    data['executable'] = executable;
    data['lamports'] = lamports;
    data['owner'] = owner;
    data['rentEpoch'] = rentEpoch;
    return data;
  }
}

class Data {
  Parsed? parsed;
  String? program;
  int? space;

  Data({this.parsed, this.program, this.space});

  Data.fromJson(Map<String, dynamic> json) {
    parsed = json['parsed'] != null ? Parsed.fromJson(json['parsed']) : null;
    program = json['program'];
    space = json['space'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    if (parsed != null) {
      data['parsed'] = parsed!.toJson();
    }
    data['program'] = program;
    data['space'] = space;
    return data;
  }
}

class Parsed {
  Info? info;
  String? type;

  Parsed({this.info, this.type});

  Parsed.fromJson(Map<String, dynamic> json) {
    info = json['info'] != null ? Info.fromJson(json['info']) : null;
    type = json['type'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    if (info != null) {
      data['info'] = info!.toJson();
    }
    data['type'] = type;
    return data;
  }
}

class Info {
  bool? isNative;
  String? mint;
  String? owner;
  RentExemptReserve? rentExemptReserve;
  String? state;
  TokenAmount? tokenAmount;

  Info(
      {this.isNative,
      this.mint,
      this.owner,
      this.rentExemptReserve,
      this.state,
      this.tokenAmount});

  Info.fromJson(Map<String, dynamic> json) {
    isNative = json['isNative'];
    mint = json['mint'];
    owner = json['owner'];
    rentExemptReserve = json['rentExemptReserve'] != null
        ? RentExemptReserve.fromJson(json['rentExemptReserve'])
        : null;
    state = json['state'];
    tokenAmount = json['tokenAmount'] != null
        ? TokenAmount.fromJson(json['tokenAmount'])
        : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['isNative'] = isNative;
    data['mint'] = mint;
    data['owner'] = owner;
    if (rentExemptReserve != null) {
      data['rentExemptReserve'] = rentExemptReserve!.toJson();
    }
    data['state'] = state;
    if (tokenAmount != null) {
      data['tokenAmount'] = tokenAmount!.toJson();
    }
    return data;
  }
}

class RentExemptReserve {
  String? amount;
  int? decimals;
  double? uiAmount;
  String? uiAmountString;

  RentExemptReserve(
      {this.amount, this.decimals, this.uiAmount, this.uiAmountString});

  RentExemptReserve.fromJson(Map<String, dynamic> json) {
    amount = json['amount'];
    decimals = json['decimals'];
    uiAmount = json['uiAmount'];
    uiAmountString = json['uiAmountString'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['amount'] = amount;
    data['decimals'] = decimals;
    data['uiAmount'] = uiAmount;
    data['uiAmountString'] = uiAmountString;
    return data;
  }
}

class TokenAmount {
  String? amount;
  int? decimals;
  double? uiAmount;
  String? uiAmountString;

  TokenAmount({this.amount, this.decimals, this.uiAmount, this.uiAmountString});

  TokenAmount.fromJson(Map<String, dynamic> json) {
    amount = json['amount'];
    decimals = json['decimals'];
    uiAmount = json['uiAmount'];
    uiAmountString = json['uiAmountString'];
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['amount'] = amount;
    data['decimals'] = decimals;
    data['uiAmount'] = uiAmount;
    data['uiAmountString'] = uiAmountString;
    return data;
  }
}
